@@ -56,6 +56,8 @@ class r_vector {
   typedef T* pointer;
   typedef T& reference;
 
+  using underlying_type = typename traits::get_underlying_type<T>::type;
+
   r_vector() noexcept = default;
 
   r_vector(SEXP data);
@@ -173,7 +175,7 @@ class r_vector {
     void fill_buf(R_xlen_t pos);
 
     R_xlen_t pos_;
-    std::array<T, 64 * 64> buf_;
+    std::array<underlying_type, 64 * 64> buf_;
     R_xlen_t block_start_ = 0;
     R_xlen_t length_ = 0;
   };
@@ -193,10 +195,10 @@ class r_vector {
   SEXP data_ = R_NilValue;
   SEXP protect_ = R_NilValue;
   bool is_altrep_ = false;
-  T* data_p_ = nullptr;
+  underlying_type* data_p_ = nullptr;
   R_xlen_t length_ = 0;
 
-  static T* get_p(bool is_altrep, SEXP data);
+  static underlying_type* get_p(bool is_altrep, SEXP data);
 
   static SEXP valid_type(SEXP data);
 
@@ -216,6 +218,8 @@ class r_vector : public cpp11::r_vector<T> {
 
   // These are necessary because type names are not directly accessible in
   // template inheritance
+  using typename cpp11::r_vector<T>::underlying_type;
+
   using cpp11::r_vector<T>::data_;
   using cpp11::r_vector<T>::data_p_;
   using cpp11::r_vector<T>::is_altrep_;
@@ -228,11 +232,11 @@ class r_vector : public cpp11::r_vector<T> {
    private:
     const SEXP data_;
     const R_xlen_t index_;
-    T* const p_;
+    underlying_type* const p_;
     bool is_altrep_;
 
    public:
-    proxy(SEXP data, const R_xlen_t index, T* const p, bool is_altrep);
+    proxy(SEXP data, const R_xlen_t index, underlying_type* const p, bool is_altrep);
 
     proxy& operator=(const T& rhs);
     proxy& operator+=(const T& rhs);
@@ -572,9 +576,9 @@ inline typename cpp11::r_vector<T>::const_iterator cpp11::r_vector<T>::find(
 template <typename T>
 inline T r_vector<T>::const_iterator::operator*() const {
   if (data_->is_altrep()) {
-    return buf_[pos_ - block_start_];
+    return static_cast<T>(buf_[pos_ - block_start_]);
   } else {
-    return data_->data_p_[pos_];
+    return static_cast<T>(data_->data_p_[pos_]);
   }
 }
 
@@ -598,13 +602,17 @@ inline T r_vector<T>::operator[](size_type pos) const {
 namespace writable {
 
 template <typename T>
-r_vector<T>::proxy::proxy(SEXP data, const R_xlen_t index, T* const p, bool is_altrep)
+r_vector<T>::proxy::proxy(SEXP data, const R_xlen_t index,
+                          typename r_vector<T>::underlying_type* const p, bool is_altrep)
     : data_(data), index_(index), p_(p), is_altrep_(is_altrep) {}
 
 template <typename T>
 inline typename r_vector<T>::proxy r_vector<T>::iterator::operator*() const {
   if (data_.is_altrep()) {
-    return proxy(data_.data(), pos_, const_cast<T*>(&buf_[pos_ - block_start_]), true);
+    return proxy(
+        data_.data(), pos_,
+        const_cast<typename r_vector<T>::underlying_type*>(&buf_[pos_ - block_start_]),
+        true);
   } else {
     return proxy(data_.data(), pos_,
                  data_.data_p_ != nullptr ? &data_.data_p_[pos_] : nullptr, false);
